@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018,2019 Michael Samoglyadov
+# Copyright 2018-2021 Michael Samoglyadov
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ from pickle import dumps
 
 __all__ = ['Differ', 'Iterator', 'Patcher', 'diff', 'patch']
 
-__version__ = '0.6'
+__version__ = '0.7'
 __author__ = 'Michael Samoglyadov'
 __license__ = 'Apache License, Version 2.0'
 __website__ = 'https://github.com/mr-mixas/Nested-Diff.py'
@@ -132,29 +132,37 @@ class Differ(object):
 
         This method is a dispatcher and calls registered diff method for each
         diffed values pair according to their type. `diff__default` called for
-        non-registered types. Args and kwargs passed to called method as is.
+        unequal and not registered types. Args and kwargs passed to called
+        method as is.
 
         :param a: First object to diff.
         :param b: Second object to diff.
 
         """
-        if self.__diff_method is not None and hasattr(a, self.__diff_method):
-            return getattr(a, self.__diff_method)(
-                b,
-                A=self.op_a,
-                N=self.op_n,
-                O=self.op_o,  # noqa: E741
-                R=self.op_r,
-                U=self.op_u,
-                trimR=self.op_trim_r,
-                diff_method=self.__diff_method,
-            )
+        if self.__diff_method is not None:
+            try:
+                method = a.__getattribute__(self.__diff_method)
+            except AttributeError:
+                pass
+            else:
+                return method(
+                    b,
+                    A=self.op_a,
+                    N=self.op_n,
+                    O=self.op_o,  # noqa: E741
+                    R=self.op_r,
+                    U=self.op_u,
+                    trimR=self.op_trim_r,
+                    diff_method=self.__diff_method,
+                )
 
         if a.__class__ is b.__class__:
-            if a == b:
+            # it's faster to compare pickled dumps and dig differences
+            # afterwards than recursively diff each pair of objects
+            if a is b or dumps(a, -1) == dumps(b, -1):
                 return {'U': a} if self.op_u else {}
-            else:
-                return self.get_differ(a.__class__)(a, b)
+
+            return self.get_differ(a.__class__)(a, b)
 
         return self.diff__default(a, b)
 
@@ -163,14 +171,14 @@ class Differ(object):
         Return default diff.
 
         """
-        ret = {}
+        dif = {}
 
         if self.op_n:
-            ret['N'] = b
+            dif['N'] = b
         if self.op_o:
-            ret['O'] = a
+            dif['O'] = a
 
-        return ret
+        return dif
 
     def diff_dict(self, a, b):
         """
@@ -189,23 +197,23 @@ class Differ(object):
         """
         dif = {}
 
-        for key in set(list(a) + list(b)):
-            if key in a and key in b:
-                if a[key] == b[key]:
-                    if self.op_u:
-                        dif[key] = {'U': a[key]}
-                else:
-                    subdiff = self.diff(a[key], b[key])
-                    if subdiff:
-                        dif[key] = subdiff
-
-            elif key in a:  # removed
-                if self.op_r:
-                    dif[key] = {'R': None if self.op_trim_r else a[key]}
-
-            else:  # added
+        for key in set(a).union(b):
+            try:
+                old = a[key]
+                try:
+                    new = b[key]
+                except KeyError:  # removed
+                    if self.op_r:
+                        dif[key] = {'R': None if self.op_trim_r else old}
+                    continue
+            except KeyError:  # added
                 if self.op_a:
                     dif[key] = {'A': b[key]}
+                continue
+
+            subdiff = self.diff(old, new)
+            if subdiff:
+                dif[key] = subdiff
 
         if dif:
             return {'D': dif}
@@ -278,7 +286,19 @@ class Differ(object):
     def diff_multiline(self, a, b):
         """
         Compute diff for multiline strings.
-        Highly experimental! Format may be changed at any time.
+
+        Result is a unified diff formatted as usual nested diff structure with
+        'I' tagged subdiffs to contain hunks headers.
+
+        :param a: First string to diff.
+        :param b: Second string to diff.
+
+        >>> a = 'A\nB\nC'
+        >>> b = 'A\nC'
+        >>>
+        >>> Differ(multiline_diff_context=3).diff_multiline(a, b)
+        {'D': [{'I': [0, 3, 0, 2]}, {'U': 'A'}, {'R': 'B'}, {'U': 'C'}],
+         'E': ''}
 
         """
         lines_a = a.split('\n', -1)
@@ -296,7 +316,7 @@ class Differ(object):
                 'I': [
                     group[0][1], group[-1][2],
                     group[0][3], group[-1][4],
-                ]
+                ],
             })
 
             for op, i1, i2, j1, j2 in group:
@@ -342,7 +362,8 @@ class Differ(object):
 
             elif i in a:  # removed
                 if self.op_r:
-                    dif.append({'R': None if self.op_trim_r else i})
+                    # ignore trimR opt here: value required for removal
+                    dif.append({'R': i})
 
             else:  # added
                 if self.op_a:
@@ -368,12 +389,12 @@ class Differ(object):
         >>>
 
         """
-        ret = self.diff_list(a, b)
+        dif = self.diff_list(a, b)
 
-        if 'D' in ret:
-            ret['D'] = tuple(ret['D'])
+        if 'D' in dif:
+            dif['D'] = tuple(dif['D'])
 
-        return ret
+        return dif
 
     def get_differ(self, type_):
         """
@@ -420,6 +441,7 @@ class Patcher(object):
             frozenset: self.patch_frozenset,
             list: self.patch_list,
             set: self.patch_set,
+            str: self.patch_multiline,
             tuple: self.patch_tuple,
         }
 
@@ -433,32 +455,35 @@ class Patcher(object):
         try:
             return self.__patchers[type_]
         except KeyError:
-            raise NotImplementedError("unsupported object type") from None
+            raise NotImplementedError('unsupported diff type') from None
 
     def patch(self, target, ndiff):
         """
         Return patched object.
 
-        This method is a dispatcher and calls registered patch method for each
-        patched value according to it's type. `patch__default` called for
-        non-registered types. Args and kwargs passed to called method as is.
+        This method is a dispatcher and calls apropriate patch method for
+        target value according to it's type.
 
         :param target: Object to patch.
         :param ndiff: Nested diff.
 
         """
-        if self.__patch_method is not None and \
-           hasattr(target, self.__patch_method):
-            return getattr(target, self.__patch_method)(ndiff)
+        if self.__patch_method is not None:
+            try:
+                method = target.__getattribute__(self.__patch_method)
+            except AttributeError:
+                pass
+            else:
+                return method(ndiff)
 
         if 'D' in ndiff:
             return self.get_patcher(
-                ndiff.get('E', ndiff['D']).__class__
+                ndiff['E' if 'E' in ndiff else 'D'].__class__,
             )(target, ndiff)
         elif 'N' in ndiff:
             return ndiff['N']
-        else:
-            return target
+
+        return target
 
     def patch_dict(self, target, ndiff):
         """
@@ -506,6 +531,41 @@ class Patcher(object):
 
         return target
 
+    def patch_multiline(self, target, ndiff):
+        """
+        Return patched multiline string.
+
+        Unlike GNU patch, this algorithm does not implement any heuristics and
+        patch target in straightforward way: get position from hunk header and
+        apply changes specified in hunk.
+
+        :param target: string to patch.
+        :param ndiff: Nested diff.
+
+        """
+        offset = 0
+        target = target.split('\n', -1)
+
+        for subdiff in ndiff['D']:
+            if 'I' in subdiff:  # hunk started
+                idx = subdiff['I'][0] + offset
+            elif 'A' in subdiff:
+                target.insert(idx, subdiff['A'])
+                offset = offset + 1
+                idx = idx + 1
+            elif 'R' in subdiff:
+                if target.pop(idx) != subdiff['R']:
+                    raise ValueError('Removing line does not match')
+                offset = offset - 1
+            elif 'U' in subdiff:
+                if target[idx] != subdiff['U']:
+                    raise ValueError('Unchanged line does not match')
+                idx = idx + 1
+            else:
+                raise ValueError('Unsupported operation')
+
+        return '\n'.join(target)
+
     @staticmethod
     def patch_set(target, ndiff):
         """
@@ -569,95 +629,102 @@ class Iterator(object):
         """
         self.sort_keys = sort_keys
 
-        self.__iters = {
-            dict: self._iter_mapping,
-            list: self._iter_sequence,
-            tuple: self._iter_sequence,
+        self.__iterators = {
+            dict: self.iterate_mapping_diff,
+            list: self.iterate_sequence_diff,
+            tuple: self.iterate_sequence_diff,
         }
 
-    def _iter_mapping(self, value):
+    @staticmethod
+    def iterate__default(ndiff):
         """
-        Iterate over dict-like objects.
+        Yield final diff (do not iterate deeper).
 
-        :param value: mapping.
+        :param ndiff: nested diff.
 
         """
-        items = sorted(value.items()) if self.sort_keys else value.items()
-        type_ = value.__class__
+        yield ndiff, None, None
 
-        for key, val in items:
-            yield type_, key, val
+    def iterate_mapping_diff(self, ndiff):
+        """
+        Iterate over dict-like nested diffs.
+
+        :param ndiff: nested diff.
+
+        """
+        items = ndiff['D'].items()
+
+        for key, subdiff in sorted(items) if self.sort_keys else items:
+            yield ndiff, key, subdiff
 
     @staticmethod
-    def _iter_sequence(value):
+    def iterate_sequence_diff(ndiff):
         """
-        Iterate over lists, tuples and other sequences.
+        Iterate over lists, tuples and alike nedsted diffs.
 
-        :param value: sequence.
+        :param ndiff: nested diff.
 
         """
         idx = 0
-        type_ = value.__class__
 
-        for item in value:
+        for item in ndiff['D']:
             if 'I' in item:
                 idx = item['I']
 
-            yield type_, idx, item
+            yield ndiff, idx, item
 
             idx += 1
 
-    def get_iter(self, value):
+    def get_iterator(self, ndiff):
         """
-        Return apropriate iterator for passed diff value.
+        Return apropriate iterator for passed nested diff.
+
+        :param ndiff: nested diff.
 
         """
+        if 'E' in ndiff:
+            return self.iterate__default(ndiff)
+
         try:
-            return self.__iters[value.__class__](value)
+            return self.__iterators[ndiff['D'].__class__](ndiff)
         except KeyError:
-            raise NotImplementedError from None
+            return self.iterate__default(ndiff)
 
-    def set_iter(self, type_, method):
+    def set_iterator(self, type_, method):
         """
-        Set generator for specified data type.
+        Set generator for specified nested diff type.
 
-        :param type_: data type.
+        :param type_: type.
         :param method: method.
 
-        Generator should yield tuples with three items: container_type, pointer
-        and subdiff.
+        Generator should yield tuples with three items: diff, key, and
+        subdiff for this key.
 
         """
-        self.__iters[type_] = method
+        self.__iterators[type_] = method
 
-    def iterate(self, ndiff, depth=0):
+    def iterate(self, ndiff):
         """
-        Return tuples with depth, container_type, pointer and subdiff for each
-        nested diff.
+        Yield tuples with diff, key and subdiff for each nested diff.
 
-        :param ndiff: Nested diff.
+        :param ndiff: nested diff.
 
         """
-        stack = [((None, None, _) for _ in (ndiff,))]
+        stack = [self.get_iterator(ndiff)]
 
-        while True:
+        while stack:
             try:
-                container_type, pointer, subdiff = next(stack[-1])
+                ndiff, key, subdiff = next(stack[-1])
             except StopIteration:
                 stack.pop()
+                continue
 
-                if stack:
-                    depth -= 1
-                    continue
-                else:
-                    break
+            yield ndiff, key, subdiff
 
-            yield depth, container_type, pointer, subdiff
+            if subdiff is None:
+                continue
 
-            if 'D' in subdiff:
-                if 'E' not in subdiff:
-                    stack.append(self.get_iter(subdiff['D']))
-                    depth += 1
+            stack.append(self.get_iterator(subdiff))
 
 
 def diff(a, b, **kwargs):
