@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2019 Michael Samoglyadov
+# Copyright 2019,2020,2021 Michael Samoglyadov
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,10 +31,10 @@ class App(object):
 
     """
     default_ifmt = 'auto'
-    default_ofmt = 'json'
+    default_ofmt = 'auto'
 
-    supported_ifmts = ('auto', 'json', 'yaml')
-    supported_ofmts = ('json', 'yaml')
+    supported_ifmts = ('auto', 'ini', 'json', 'yaml')
+    supported_ofmts = ('auto', 'ini', 'json', 'yaml')
 
     version = nested_diff.__version__
 
@@ -44,11 +44,6 @@ class App(object):
         self.argparser = self.get_argparser(description=self.__doc__)
         self.args = self.argparser.parse_args(args=args)
 
-        self.dumper = self.get_dumper(
-            self.args.ofmt,
-            **self._decode_fmt_opts(self.args.ofmt_opts)
-        )
-
     @staticmethod
     def _decode_fmt_opts(opts):
         if opts is None:
@@ -57,16 +52,28 @@ class App(object):
         import json
         return json.loads(opts)
 
-    def dump(self, file_, data):
-        self.dumper.dump(file_, data)
+    def dump(self, file_, data, fmt):
+        self.get_dumper(
+            fmt,
+            **self._decode_fmt_opts(self.args.ofmt_opts)  # noqa C815
+        ).dump(file_, data)
 
     def get_argparser(self, description=None):
-        parser = argparse.ArgumentParser(description=description)
+        return argparse.ArgumentParser(
+            description=description,
+            parents=(
+                self.get_optional_args_parser(),
+                self.get_positional_args_parser(),
+            ),
+        )
+
+    def get_optional_args_parser(self):
+        parser = argparse.ArgumentParser(add_help=False)
         parser.add_argument(
             '--version',
             action='version',
             version='%(prog)s {}'.format(self.version),
-            help='print version and exit'
+            help='print version and exit',
         )
 
         parser.add_argument(
@@ -74,15 +81,14 @@ class App(object):
             type=str,
             default=self.default_ifmt,
             choices=sorted(self.supported_ifmts),
-            help='input files format; "' + self.default_ifmt +
-                 '" used by default',
+            help='input files format; "%(default)s" is used by default',
         )
 
         parser.add_argument(
             '--ifmt-opts',
             metavar='JSON',
             type=str,
-            help='input files format options',
+            help='input files format options (JSON string)',
         )
 
         parser.add_argument(
@@ -90,32 +96,21 @@ class App(object):
             type=str,
             default=self.default_ofmt,
             choices=sorted(self.supported_ofmts),
-            help='output files format; "' + self.default_ofmt +
-                 '" used by default',
+            help='output files format; "%(default)s" is used by default',
         )
 
         parser.add_argument(
             '--ofmt-opts',
             metavar='JSON',
             type=str,
-            help='output files format options',
+            help='output files format options (JSON string)',
         )
 
-        for name, opts in self.get_argparser_positional_args():
-            parser.add_argument(name, **opts)
-
         return parser
 
     @staticmethod
-    def get_argparser_positional_args():
-        """
-        Yield tuples (name, opts) for each positional argument.
-
-        It's almost impossible to remove positional args from argparser, so
-        we use method here to be able to override it in subclasses.
-
-        """
-        return ()
+    def get_positional_args_parser():
+        return argparse.ArgumentParser(add_help=False)
 
     @staticmethod
     def get_dumper(fmt, **kwargs):
@@ -123,6 +118,8 @@ class App(object):
             return JsonDumper(**kwargs)
         elif fmt == 'yaml':
             return YamlDumper(**kwargs)
+        elif fmt == 'ini':
+            return IniDumper(**kwargs)
 
         raise RuntimeError('Unsupported output format: ' + fmt)
 
@@ -148,6 +145,8 @@ class App(object):
             return JsonLoader(**kwargs)
         elif fmt == 'yaml':
             return YamlLoader(**kwargs)
+        elif fmt == 'ini':
+            return IniLoader(**kwargs)
 
         raise RuntimeError('Unsupported input format: ' + fmt)
 
@@ -260,6 +259,26 @@ class JsonLoader(Loader):
         return self.decoder.decode(data)
 
 
+class IniDumper(Dumper):
+    """
+    INI dumper
+
+    """
+    def __init__(self, **kwargs):
+        super().__init__()
+
+        import configparser
+        import io
+        self.encoder = configparser.ConfigParser(**self.get_opts(kwargs))
+        self.stringio = io.StringIO()
+
+    def encode(self, data):
+        self.encoder.read_dict(data)
+        self.encoder.write(self.stringio)
+
+        return self.stringio.getvalue()
+
+
 class IniLoader(Loader):
     """
     INI loader
