@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2019 Michael Samoglyadov
+# Copyright 2019-2021 Michael Samoglyadov
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,9 +30,6 @@ class App(nested_diff.cli.App):
     Diff tool for nested data structures
 
     """
-    default_ifmt = 'auto'
-    default_ofmt = 'auto'
-
     supported_ifmts = ('auto', 'ini', 'json', 'yaml')
     supported_ofmts = ('auto', 'json', 'term', 'text', 'yaml')
 
@@ -54,13 +51,8 @@ class App(nested_diff.cli.App):
             U=self.args.U,
         )
 
-    def get_argparser(self, description=None):
-        parent = super().get_argparser()
-        parser = argparse.ArgumentParser(
-            conflict_handler='resolve',
-            description=description,
-            parents=(parent,)
-        )
+    def get_optional_args_parser(self):
+        parser = super().get_optional_args_parser()
 
         parser.add_argument(
             '--text-ctx',
@@ -68,7 +60,8 @@ class App(nested_diff.cli.App):
             metavar='NUM',
             type=int,
             help='amount of context lines for multiline strings diffs; '
-                 'negative value will disable multiline diffs, default is 3'
+                 'negative value will disable multiline diffs, default is '
+                 '"%(default)s"',
         )
 
         parser.add_argument(
@@ -82,9 +75,9 @@ class App(nested_diff.cli.App):
         parser.add_argument('-A', type=int, choices=(0, 1), default=1,
                             help='show added items; enabled by default')
         parser.add_argument('-N', type=int, choices=(0, 1), default=1,
-                            help='show item\'s new values; enabled by default')
+                            help="show item's new values; enabled by default")
         parser.add_argument('-O', type=int, choices=(0, 1), default=1,
-                            help='show item\'s old values; enabled by default')
+                            help="show item's old values; enabled by default")
         parser.add_argument('-R', type=int, choices=(0, 1), default=1,
                             help='Show removed items; enabled by default')
         parser.add_argument('-U', type=int, choices=(0, 1), default=0,
@@ -92,10 +85,13 @@ class App(nested_diff.cli.App):
 
         return parser
 
-    @staticmethod
-    def get_argparser_positional_args():
-        yield 'file1', {'type': argparse.FileType()}
-        yield 'file2', {'type': argparse.FileType()}
+    def get_positional_args_parser(self):
+        parser = super().get_positional_args_parser()
+
+        parser.add_argument('file1', type=argparse.FileType())
+        parser.add_argument('file2', type=argparse.FileType())
+
+        return parser
 
     def get_dumper(self, fmt, **kwargs):
         if fmt == 'auto':
@@ -111,12 +107,6 @@ class App(nested_diff.cli.App):
 
         return super().get_dumper(fmt, **kwargs)
 
-    def get_loader(self, fmt, **kwargs):
-        if fmt == 'ini':
-            return nested_diff.cli.IniLoader(**kwargs)
-
-        return super().get_loader(fmt, **kwargs)
-
     def run(self):
         diff = self.diff(
             self.load(self.args.file1),
@@ -124,7 +114,7 @@ class App(nested_diff.cli.App):
         )
         exit_code = 0 if not diff or 'U' in diff else 1
 
-        self.dump(self.args.out, diff)
+        self.dump(self.args.out, diff, self.args.ofmt)
 
         return exit_code
 
@@ -142,15 +132,15 @@ class AbstractFmtDumper(nested_diff.cli.Dumper):
 class TermDumper(AbstractFmtDumper):
     def __init__(self, **kwargs):
         super().__init__()
-        import nested_diff.fmt
-        self.encoder = nested_diff.fmt.TermFormatter(**self.get_opts(kwargs))
+        from nested_diff import fmt
+        self.encoder = fmt.TermFormatter(**self.get_opts(kwargs))
 
 
 class TextDumper(AbstractFmtDumper):
     def __init__(self, **kwargs):
         super().__init__()
-        import nested_diff.fmt
-        self.encoder = nested_diff.fmt.TextFormatter(**self.get_opts(kwargs))
+        from nested_diff import fmt
+        self.encoder = fmt.TextFormatter(**self.get_opts(kwargs))
 
 
 def cli():
