@@ -30,34 +30,41 @@ class AbstractFormatter(nested_diff.Iterator):
         self,
         indent='  ',
         line_separator='\n',
-        **kwargs
+        header='',
+        footer='',
+        **kwargs  # noqa C816
     ):
         super().__init__(**kwargs)
 
         self.indent = indent
         self.line_separator = line_separator
+        self.header = self.get_header() if header is None else header
+        self.footer = self.get_footer() if footer is None else footer
 
-        self.open_tokens = {
+        self.obj_prefix = {
             dict: '{',
             list: '[',
             tuple: '(',
         }
-        self.close_tokens = {
+        self.obj_suffix = {
             dict: '}',
             list: ']',
             tuple: ')',
         }
 
-        self.diff_key_tokens = {
+        self.key_line_prefix = {
             'A': '+ ',
             'D': '  ',
-            'I': '  ',
-            'N': '+ ',
-            'O': '- ',
+            'N': '  ',
+            'O': '  ',
             'R': '- ',
             'U': '  ',
         }
-        self.diff_value_tokens = self.diff_key_tokens.copy()
+        self.val_line_prefix = self.key_line_prefix.copy()
+        self.val_line_prefix['E'] = '# '
+        self.val_line_prefix['I'] = '  '
+        self.val_line_prefix['O'] = '- '
+        self.val_line_prefix['N'] = '+ '
 
         self.tags = (  # diff tags to format, sequence is important
             'D',
@@ -68,6 +75,20 @@ class AbstractFormatter(nested_diff.Iterator):
             'U',
         )
 
+    def get_header(self):
+        """
+        Return header for formatted diff.
+
+        """
+        return ''
+
+    def get_footer(self):
+        """
+        Return footer for formatted diff.
+
+        """
+        return ''
+
     @staticmethod
     def get_unified_diff_range(start, stop):
         """
@@ -79,10 +100,7 @@ class AbstractFormatter(nested_diff.Iterator):
         if length > 1:
             return '{},{}'.format(start + 1, length)
 
-        if length == 1:
-            return str(start + 1)
-
-        return '{},{}'.format(start, length)
+        return str(start + 1)
 
     def format(self, diff, **kwargs):
         """
@@ -91,20 +109,6 @@ class AbstractFormatter(nested_diff.Iterator):
         """
         return ''.join(self.emit_tokens(diff, **kwargs))
 
-    def get_open_token(self, type_):
-        """
-        Return open token for specified container's type
-
-        """
-        return self.open_tokens[type_]
-
-    def get_close_token(self, type_):
-        """
-        Return close token for specified container's type
-
-        """
-        return self.close_tokens[type_]
-
     @staticmethod
     def repr_key(key):
         """
@@ -121,24 +125,10 @@ class AbstractFormatter(nested_diff.Iterator):
         """
         return val.__repr__()
 
-    def set_open_token(self, type_, token):
-        """
-        Set open token for specified container's type
-
-        """
-        self.open_tokens[type_] = token
-
-    def set_close_token(self, type_, token):
-        """
-        Set close token for specified container's type
-
-        """
-        self.close_tokens[type_] = token
-
 
 class TextFormatter(AbstractFormatter):
     """
-    Produce human friendly text diff representation with indenting formatting.
+    Produce human friendly text diff with indenting formatting.
 
     """
     def __init__(self, *args, **kwargs):
@@ -155,11 +145,13 @@ class TextFormatter(AbstractFormatter):
         Yield unified diff for multiline strings.
 
         """
+        indent = self.indent * depth
+
         for subdiff in diff['D']:
             for tag in ('I', 'R', 'A', 'U'):
                 if tag in subdiff:
-                    yield self.diff_value_tokens[tag]
-                    yield self.indent * depth
+                    yield self.val_line_prefix[tag]
+                    yield indent
                     if tag == 'I':
                         yield '@@'
                         yield ' -' + self.get_unified_diff_range(
@@ -176,24 +168,74 @@ class TextFormatter(AbstractFormatter):
         Yield tokens for set's and frozenset's diff
 
         """
-        yield self.diff_key_tokens['D']
+        indent = self.indent * depth
+
+        for subdiff in diff['D']:
+            for tag in ('R', 'A', 'U'):
+                if tag in subdiff:
+                    yield self.val_line_prefix[tag]
+                    yield indent
+                    yield self.repr_value(subdiff[tag])
+                    yield self.line_separator
+                    break
+
+    def emit_type_header(self, diff, depth=0):
+        yield self.val_line_prefix['E']
         yield self.indent * depth
         yield '<'
         yield diff['E'].__class__.__name__
         yield '>'
         yield self.line_separator
 
-        depth += 1
+    def emit_tokens(self, diff, depth=0):
+        """
+        Yield formatted diff token by token
 
-        for subdiff in diff['D']:
-            for tag in ('R', 'A', 'U'):
+        """
+        yield self.header
+
+        stack = [self.get_iterator(diff)]
+
+        while stack:
+            try:
+                diff, key, subdiff = next(stack[-1])
+            except StopIteration:
+                stack.pop()
+                depth -= 1
+                continue
+
+            # emit value
+            if 'E' in diff:
+                yield from self.emit_type_header(diff, depth=depth)
+                yield from self.get_emitter(diff, depth=depth)
+                continue
+
+            if subdiff is None:
+                for tag in self.tags:
+                    if tag in diff:
+                        yield self.val_line_prefix[tag]
+                        yield self.indent * depth
+                        yield self.repr_value(diff[tag])
+                        yield self.line_separator
+                continue
+
+            # emit key
+            diff_type = diff['D'].__class__
+            for tag in self.tags:
                 if tag in subdiff:
-                    yield self.diff_value_tokens[tag]
+                    yield self.key_line_prefix[tag]
                     yield self.indent * depth
-                    yield self.repr_value(subdiff[tag])
+                    yield self.obj_prefix[diff_type]
+                    yield self.repr_key(key)
+                    yield self.obj_suffix[diff_type]
                     yield self.line_separator
                     break
 
+            depth += 1
+            stack.append(self.get_iterator(subdiff))
+
+        yield self.footer
+
     def get_emitter(self, diff, depth=0):
         """
         Return apropriate tokens emitter for diff extention.
@@ -204,49 +246,6 @@ class TextFormatter(AbstractFormatter):
         except KeyError:
             raise NotImplementedError from None
 
-    def emit_tokens(self, diff, depth=0, header='', footer=''):
-        """
-        Yield formatted diff token by token
-
-        """
-        if header:
-            yield header
-            yield self.line_separator
-
-        key_tag = 'D'
-
-        for depth, container_type, pointer, diff in self.iterate(
-                diff, depth=depth):
-
-            for tag in self.tags:
-                if tag in diff:
-                    # key/index
-                    if key_tag is None:
-                        key_tag = 'D' if tag in ('O', 'N') else tag
-                        yield self.diff_key_tokens[key_tag]
-                        yield self.indent * (depth - 1)
-                        yield self.get_open_token(container_type)
-                        yield self.repr_key(pointer)
-                        yield self.get_close_token(container_type)
-                        yield self.line_separator
-
-                    # value
-                    if tag == 'D':
-                        if 'E' in diff:
-                            yield from self.get_emitter(diff, depth=depth)
-                        break
-
-                    yield self.diff_value_tokens[tag]
-                    yield self.indent * depth
-                    yield self.repr_value(diff[tag])
-                    yield self.line_separator
-
-            key_tag = None
-
-        if footer:
-            yield footer
-            yield self.line_separator
-
 
 class TermFormatter(TextFormatter):
     """
@@ -258,20 +257,12 @@ class TermFormatter(TextFormatter):
 
         self.line_separator = '\033[0m' + self.line_separator
 
-        self.diff_key_tokens = {
-            'A': '\033[1;32m+ ',
-            'D': '  ',
-            'N': '\033[1;32m+ ',
-            'O': '\033[1;31m- ',
-            'R': '\033[1;31m- ',
-            'U': '  ',
-        }
-        self.diff_value_tokens = {
-            'A': '\033[32m+ ',
-            'D': '  ',
-            'I': '\033[35m  ',
-            'N': '\033[32m+ ',
-            'O': '\033[31m- ',
-            'R': '\033[31m- ',
-            'U': '  ',
-        }
+        self.key_line_prefix['A'] = '\033[1;32m' + self.key_line_prefix['A']
+        self.key_line_prefix['R'] = '\033[1;31m' + self.key_line_prefix['R']
+
+        self.val_line_prefix['A'] = '\033[32m' + self.val_line_prefix['A']
+        self.val_line_prefix['E'] = '\033[34m' + self.val_line_prefix['E']
+        self.val_line_prefix['I'] = '\033[35m' + self.val_line_prefix['I']
+        self.val_line_prefix['N'] = '\033[32m' + self.val_line_prefix['N']
+        self.val_line_prefix['O'] = '\033[31m' + self.val_line_prefix['O']
+        self.val_line_prefix['R'] = '\033[31m' + self.val_line_prefix['R']
